;-------------------------------------------------------------------------------------
; video_postprocess.ahk - an AutoHotKey script to postprocess a video after capture
;                         in DSLR Remote Pro
;-------------------------------------------------------------------------------------
; This script can rotate, resize and add an overlay and soundtrack to a video
; captured using DSLR Remote Pro v3.11.3 onwards. It uses ffmpeg to modify the
; video.
;
; Usage: set the "Delay before playback" to 20 seconds in the video booth
; settings in DSLR Remote Pro and then double click on this script to run it.
;
; Command line usage to process a single video by reading the XML summary file:
; Open a command prompt window, cd to the folder containing this
; script and then run ot with the pathname of the video booth XML summary file
; e.g. video_postprocess.ahk C:\Users\chris\Documents\PhotoboothImages\2018-10-31\IMG_0001.xml
;
; Add an optional overlay by placing a PNG file named video_overlay.png in the photo booth images folder
; Add an optional soundtrack by placing an audio file named video_soundtrack.mp3 or video_soundtrack.wav
; in the photo booth images folder
;
; Edit this script to change the following variables:
; copyFolder - the name of the subfolder in which to save a copy of the original video
; rotateDegreesClockwise - optional rotation in 90 degree intervals
; videoHeight - height of the video (after rotation). Set this to 0 to disable resizing
;
; FFmpeg can be downloaded from https://www.ffmpeg.org/
;
; AutoHotKey is free and can be downloaded from:
; http://www.autohotkey.com
;
; This script comes with no warranty or support whatsoever and may be freely
; copied or modified as required.
;
; Written by Chris Breeze, www.breezesys.com
; 20180816: Initial version
; 20181101: Added video preset setting and the ability to process XML files from cmd line
; 20190329: Added option to include audio from original video
;----------------------------------------------------------------------------------------
#singleinstance force
SetTitleMatchMode,2
StringCaseSense, Off

; subfolder in which to save a copy of the original video
copyFolder := "originals"

; rotate video in 90 degree increments, valid values: 0, 90, 180, 270
rotateDegreesClockwise := 0

; height of the video (after rotation). Set this to 0 to disable resizing
videoHeight := 600

; ffmpeg video preset, choose from: veryslow, slow, fast, veryfast and ultrafast
videoPreset := "fast"

; include audio from original video
includeOriginalAudio := 0

; set this to 1 to display debug info (0 to disable debug)
showDebugMessages := 0

;----------- Don't change anything below here ----------------


; make sure videoHeight is an even number
if Mod(videoHeight, 2)
{
	videoHeight++
}

ffmpeg = %A_ScriptDir%\ffmpeg.exe
ifNotExist,%ffmpeg%
{
	MsgBox ERROR: ffmpeg.exe not found.`nPlease make sure ffmpeg.exe is in the same folder as this script:`n%A_ScriptDir%
	exitapp
}

if showDebugMessages
{
	; display a debug window at the top of the screen
	Gui +LastFound +AlwaysOnTop +ToolWindow  ; +ToolWindow avoids a taskbar button and an alt-tab menu item.
	Gui, Color, cBlack
	Gui, Font, s12 
	Gui, Add, Text, vStatusText cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Add, Text, vStatusText2 cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Show,x100 y0 NoActivate,%A_ScriptName%  ; NoActivate avoids deactivating the currently active window.
	GuiControl,,StatusText,Please wait...
	GuiControl,,StatusText2,
}

; look for XML file in the command line
if %0%
{
	path = %1%
	SplitPath, path,,, ext
	if (ext = "mp4")
	{
		video = %1%
		if %0% > 1
		{
			screenDir = %2%
		}
		else
		{
			screenDir := "."
		}
	}
	else if (ext = "xml")
	{
		IfNotExist, %1%
		{
			MsgBox ERROR: XML file not found: %1% 
			exitapp
		}
		; read XML file, process video and exit
		FileRead, Contents, %1%
		RegExMatch(Contents, "<video_information>", video) 
		RegExMatch(Contents, "<photobooth_images_folder>(.*)</photobooth_images_folder>", photoboothFolder) 
		RegExMatch(Contents, "<download_folder>(.*)</download_folder>", downloadFolder) 
		RegExMatch(Contents, "<video_file>(.*)</video_file>", file) 

		if not Contents contains <video_information>
		{
			MsgBox ERROR: not a video booth summary file: %1%
			exitapp
		}
		if photoboothFolder < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <photobooth_images_folder> tag
			exitapp
		}
		if downloadFolder < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <download_folder> tag
			exitapp
		}
		if file < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <video_file> tag
			exitapp
		}

		screenDir = %photoboothFolder1%
		video = %downloadFolder1%\%file1%
	}
	else
	{
		MsgBox XML or MP4 image not found 
		exitapp
	}

	IfNotExist, %video%
	{
		MsgBox ERROR: video file not found: %video% 
		exitapp
	}
	IfNotExist, %screenDir%
	{
		MsgBox ERROR: Screen folder not found: %screenDir% 
		exitapp
	}
	showDebugMessages = 1
	ProcessVideo(video, screenDir)
	exitapp
}

; loop waiting for video files to process
DebugMessage(1,"Window title")
Loop
{
	IfWinExist, Breeze Systems Photobooth - 
	{
		WinGetTitle, Title, Breeze Systems Photobooth - 
		if title contains video_processing
		{
			RegExMatch(title, "\s-\s(.*)", screen) 
			SplitPath, screen1, screenFilename, screenDir
			DebugMessage(1, "Screen folder: " screenDir)
			ControlGetText, msg, Static2, Breeze Systems Photobooth
			RegExMatch(msg, ",\s*movie=(.*)", video) 
			IfExist, %video1%
			{
				ProcessVideo(video1, screenDir)

				; tell photo booth the processed video is ready
				WinActivate, Breeze Systems Photobooth -
				Send ^z
				DebugMessage(2, "Sending Ctrl+Z to photo booth...")
				Sleep 1000
			}
			else
			{
				DebugMessage(2, "Waiting for movie to download...")
			}
		}
		else
		{
			DebugMessage(1, Title)
			DebugMessage(2, "Waiting for video_processing screen...")
		}
	}
	else
	{
		IfWinExist, DSLR Remote Pro for Windows -
		{
			DebugMessage(1, "Not running in photo booth mode")
		}
		else
		{
			DebugMessage(1, "DSLR Remote Pro is not running")
		}
		DebugMessage(2, "")
	}
	Sleep 100	; sleep for 1/10th sec to avoid hogging the processor
}

ProcessVideo(video, screenFolder)
{
	global ffmpeg, copyFolder, videoHeight, rotateDegreesClockwise
	global videoPreset, showDebugMessages

	DebugMessage(2, "Processing " video)

	rotate :=
	if rotateDegreesClockwise = 90
	{
		rotate := "transpose=1"
	}
	else if rotateDegreesClockwise = 180
	{
		rotate := "transpose=2,transpose=2"
	}
	else if rotateDegreesClockwise = 270
	{
		rotate := "transpose=2"
	}

	; make a copy of the original video
	SplitPath,video,name,folder
	videoCopy = %folder%\%copyFolder%
	FileCreateDir,%videoCopy%
	videoCopy = %videoCopy%\%name%
	FileDelete,%videoCopy%
	FileCopy,%video%,%videoCopy%

	overlay = %screenFolder%\video_overlay.png
	music = %screenFolder%\video_soundtrack.mp3
	IfNotExist,%music%
	{
		music = %screenFolder%\video_soundtrack.wav
	}

	cmd = "%ffmpeg%" -y -i "%videoCopy%"
	filter :=
	IfExist %overlay%
	{
		; add overlay
		cmd = %cmd% -i "%overlay%"
		if videoHeight > 0
		{
			; resize video (stream 0) then add overlay (stream 1)
			if %rotate%
			{
				filter = -filter_complex "[0:v]%rotate%,scale=-2:%videoHeight%[bg],[bg][1:v]overlay=0:0"
			}
			else
			{
				filter = -filter_complex "[0:v]scale=-2:%videoHeight%[bg],[bg][1:v]overlay=0:0"
			}
		}
		else
		{
			; original video (stream 0) with overlay (stream 1)
			if %rotate%
			{
				filter = -filter_complex "[0:v]%rotate%[bg],[bg][1:v]overlay=0:0"
			}
			else
			{
				filter = -filter_complex "[0:v][1:v]overlay=0:0"
			}
		}
	}
	else if videoHeight > 0
	{
		; resize video using video filter
		if %rotate%
		{
			filter = -vf "%rotate%,scale=-2:%videoHeight%"
		}
		else
		{
			filter = -vf "scale=-2:%videoHeight%"
		}
	}
	else if %rotate%
	{
		filter = -vf "%rotate%"
	}

	IfExist %music%
	{
		; add music and truncate video to shortest clip
		cmd = %cmd% -i "%music%"
		filter = %filter% -c:a aac -shortest
	}
	else if %includeOriginalAudio%
	{
		filter = %filter% -c:a aac -shortest
	}

	;cmd = %cmd% %filter% "%video%"
	cmd = %cmd% %filter% -c:v libx264 -preset %videoPreset% "%video%"
	if showDebugMessages = 2
	{	
		MsgBox %cmd%
	}

	if showDebugMessages
	{
		runwait,%cmd%
	}
	else
	{
		runwait,%cmd%,,Hide
	}
}

; Display a debug message if showDebugMessages = 1
; num specifies the line on which to display the message (1 or 2)
; msg is the message to display
DebugMessage(num, msg)
{
	global cStatusText, cStatusText2
	global showDebugMessages
	msg := A_Hour ":" A_Min ":" A_Sec ": " msg
	if showDebugMessages
	{
		if num=1
		{
			GuiControl,,StatusText,%msg%
		}
		else
		{
			GuiControl,,StatusText2,%msg%
		}
	}
}

GuiClose:
ExitApp
